/* Plasma Data Engine dependency extractor
   Copyright (C) 2011 Kevin Kofler <kevin.kofler@chello.at>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <QCoreApplication>
#include <QTextStream>
#include <QFileInfo>
#include <QDir>

#include <cstdio>

#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <kdesktopfile.h>
#include <kconfiggroup.h>

#include "private/componentinstaller_p.h"

static QString scriptingApi(const QString &desktopFile)
{
    KDesktopFile desktop(desktopFile);
    KConfigGroup desktopGroup = desktop.desktopGroup();
    if (desktopGroup.readEntry("X-KDE-ServiceTypes", QStringList())
                      .contains("Plasma/ScriptEngine")
        || desktopGroup.readEntry("ServiceTypes", QStringList())
                         .contains("Plasma/ScriptEngine")) {
        /* Script engines are always written in C++. Their X-Plasma-API is the
           API they export, not the language they're written in. */
        return QString();
    }
    return desktopGroup.readEntry("X-Plasma-API", QString());
}

static void writeDataEngineDependencies(const QStringList &deps,
                                        const QString &desktopFile)
{
    if (!deps.isEmpty()) {
        KDesktopFile desktop(desktopFile);
        desktop.desktopGroup().writeEntry("X-Plasma-RequiredDataEngines", deps);
    }
}

int main(int argc, char **argv)
{
    KAboutData aboutData("plasma-dataengine-depextractor", QByteArray(),
                        ki18n("Plasma Data Engine dependency extractor"),
                        "2",
                        ki18n("Plasma Data Engine dependency extractor"));
    aboutData.addAuthor(ki18n("Kevin Kofler"), ki18n("Author"),
                        "kevin.kofler@chello.at");

    KCmdLineArgs::init(argc, argv, &aboutData);
    KCmdLineOptions options;
    options.add("+[path]",
                ki18n("Source path (default: .)"));
    options.add("+[file]",
                ki18n(".desktop rel. to path (default: metadata.desktop)")
               );
    KCmdLineArgs::addCmdLineOptions(options);

    QCoreApplication *app = new QCoreApplication(KCmdLineArgs::qtArgc(),
                                                 KCmdLineArgs::qtArgv());

    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    int exitCode = 0;

    QString path, desktopFile;
    int argCount = args->count();
    switch (argCount) {
        case 0:
            path = ".";
            desktopFile = "metadata.desktop";
            break;
        case 1:
            path = args->arg(0);
            desktopFile = "metadata.desktop";
            break;
        case 2:
            path = args->arg(0);
            desktopFile = args->arg(1);
            break;
        default:
        {
            QTextStream err(stderr, QIODevice::WriteOnly | QIODevice::Text);
            err << i18n("Expected at most 2 arguments, but %1 given", argCount)
                << endl;
            exitCode = 1;
            break;
        }
    }

    if (!exitCode) {
        if (QFileInfo(desktopFile).isRelative())
            desktopFile = QDir(path).absoluteFilePath(desktopFile);

        if (QFileInfo(desktopFile).exists()) {
            writeDataEngineDependencies(Plasma::ComponentInstaller::self()
                                          ->extractDataEngineDependencies(
                                              path,
                                              scriptingApi(desktopFile)),
                                        desktopFile);
        } else {
            QTextStream err(stderr, QIODevice::WriteOnly | QIODevice::Text);
            err << i18n("Desktop file \"%1\" not found", desktopFile) << endl;
            exitCode = 1;
        }
    }

    args->clear();
    delete app;
    return exitCode;
}
